Robo2.Log("Script: " + SCRIPT_FILE)


// Definition of user defined variables

Robo2.SetDialogVariable("pre_agonist_s", 5, "Time before agonist application (s)");
Robo2.SetDialogVariable("agonist_s", 15, "Agonist application time (s)");
Robo2.SetDialogVariable("recorded_washout_s", 20, "Recorded wash-out time (s)");
Robo2.SetDialogVariable("nonrecorded_washout_s", 0, "Non-recordeded wash time (s)");

Robo2.SetDialogVariable("pumpspeed", 5000, "Speed of valve pump (10 - 10000)");
Robo2.SetDialogVariable("ratio", 70, "Ratio between waste and valve pump in percent - must be greater than 70");

Robo2.SetDialogVariable("imp_steps", 10, "Maximum Total Number of Impalement Steps");
Robo2.SetDialogVariable("impstep", 50, "Single Impalement Step Distance in Micrometer");
Robo2.SetDialogVariable("minrmp", -6, "Minimum Membrane Potential after Impalement");

Robo2.SetDialogVariable("clampvoltage", -60, "Recording Voltage in mV");
Robo2.SetDialogVariable("minResponse", -100, "Minimum response in the expression test to continue the recording (nA)");
Robo2.SetDialogVariable("maxResponse", -30000, "Maximum response in the expression test to continue the recording (nA)");
Robo2.SetDialogVariable("first_compound", 2, "Valve number of the first compound used for the DRC (2-12)");
Robo2.SetDialogVariable("last_compound", 7, "Valve number of the last compound used for the DRC (2-12)");
Robo2.SetDialogVariable("reference", 4, "Valve number of the reference compound used for expression test (3-12)");


Robo2.ShowDialog();

// This opens the dialogue to change user defined variables from above - please note that changes are not permanent

//Comments for the Log window

Robo2.Log("Result of variables");
Robo2.Log(" ");
Robo2.Log("Time before agonist application: " + pre_agonist_s + " s");
Robo2.Log("Agonist application time: " + agonist_s + " s");
Robo2.Log("Recorded wash-time: " + recorded_washout_s + " s");
Robo2.Log("Non-recorded wash-time: " + nonrecorded_washout_s + " s");
Robo2.Log(" ");
Robo2.Log("Reference compound/concentration from valve " + reference);
Robo2.Log("Last coumpound used in valve " + last_compound);

//Assigning values to the built-in variables

Set_SAMPLERATE(500)

Set_AMPLIFIER_GAIN_P(1000)
Set_AMPLIFIER_GAIN_I(100)

Set_DCOFFSET_RANGE(3);
Set_DCOFFSET_DELAY(15);
Set_DCOFFSET_WAIT(5);

Set_MIN_RMP(minrmp);
Set_IMPALEMENT_STEPS(imp_steps)
Set_IMPALEMENT_STEP(impstep)
Set_IMPALEMENT_WAIT(1)

Set_MIN_INITIAL_LEAKCURRENT(-10000)
Set_MAX_INITIAL_LEAKCURRENT(500)

Set_MIN_LEAKCURRENT(-1000)
Set_MAX_LEAKCURRENT(200)

Set_LEAKCURRENT_WAIT(10)
Set_LEAKCURRENT_ATTEMPTS(3)

Set_MIN_RESISTANCE_I(100)
Set_MAX_RESISTANCE_I(1500)

Set_MIN_RESISTANCE_U(100)
Set_MAX_RESISTANCE_U(1500)

// This opens the dialogue to change values for the built-in variables

Robo2.ShowStandardDialog()



//** Initialisation - Setting sample rate and amplifier gain values
// -----------------

Robo2.SetAmplifierCoefficients(AMPLIFIER_GAIN_P, AMPLIFIER_GAIN_I);
RecDisplay.Clear();
Robo2.SetSampleRate(SAMPLERATE);
		
// Start of Oocyte loop
// ----------------------------------------------

for (var i = 0; i < Robo2.SelectedWells.Count; i++)
{
	var WellIndex = Robo2.SelectedWells[i];

	ControlDisplay.Clear();
	RecDisplay.Clear();

	// Measuring head moves to selected well
	
	Robo2.Log("_______________________");
	Robo2.Log("Starting with new oocyte in well " + Robo2.SelectedWellNames[i] );
	Robo2.Log("    ");
	
		
		Robo2.MoveToWell(WellIndex);
		Robo2.WastePumpOn(pumpspeed*ratio/100);
		Robo2.Wait(1);		
		Robo2.ZMoveToLiquid();
		Robo2.Wait(2);	

	
// Setting of axis scalings 
		
	ControlDisplay.SetXAxis(0, 30);
	ControlDisplay.SetYAxis_U(-100, 10);
	ControlDisplay.SetYAxis_I(-30, 30);

// Starting Current Clamp mode
	
	Robo2.Log("Current clamp at 0 nA");
	Robo2.SetHoldingCurrent(0);
	Robo2.SetCurrentClamp();

// Starting Control Recording for electrode offset reset and resistance test
	
		Robo2.StartControlRecording();

		// Electrode offset reset            
		Robo2.Log("Elektrode reset ...");

		if (!Robo2.DCOffsetCorrection(DCOFFSET_RANGE, DCOFFSET_DELAY, DCOFFSET_WAIT, DCOFFSET_ATTEMPTS))
		{
			continue;
		}

// Electrode resistance test
		
		if (Robo2.ResistanceCheck_I(MIN_RESISTANCE_I, MAX_RESISTANCE_I))
		{
		}
		else
		{Robo2.StopControlRecording();
			continue;
		}

		if (Robo2.ResistanceCheck_U(MIN_RESISTANCE_U, MAX_RESISTANCE_U))
		{
		}
		else
		{Robo2.StopControlRecording();
			continue;
		}

		Robo2.StopControlRecording();

// Impalement procedure

		Robo2.Log("Starting Oocyte Impalement...");
		
		Robo2.StartControlRecording();
	
		Robo2.Wait(2);
	
		ControlDisplay.SetXAxis(0, 30)
		ControlDisplay.SetYAxis_U(-60, 10)
	
		Robo2.Wait(5)
	
		if (!Robo2.Impale(MIN_RMP, IMPALEMENT_STEPS, IMPALEMENT_STEP, IMPALEMENT_WAIT))
		{
		Robo2.Log("impalement failed, " + MIN_RMP + " not reached --> next oocyte");
		Robo2.ValvePumpOff();
		Robo2.CloseAllValves();
		Robo2.WastePumpOff();
			continue;
		}

// Script continues here if impalement was successful

		Robo2.Log("Waiting 10 s for resealing of membrane ...");
		Robo2.Wait(10);
		Robo2.Log("... Final membrane potential after 10 s = " + Robo2.VALUE + "mV");
		Robo2.Log(" ");
	
		Robo2.StopControlRecording();

// Starting Voltage Clamp and leak current check routine
	
		Robo2.Log("Voltage clamp ...");
	
		ControlDisplay.SetXAxis(0, 60)
		ControlDisplay.SetYAxis_U(clampvoltage-10, clampvoltage+10)
		ControlDisplay.SetYAxis_I(MIN_INITIAL_LEAKCURRENT, MAX_INITIAL_LEAKCURRENT)
	
	
		Robo2.SetHoldingVoltage(clampvoltage);
		Robo2.SetVoltageClamp();
	
		Robo2.Log("Initial leak current check (maximum = " + MIN_INITIAL_LEAKCURRENT + " nA) ...");
	
		Robo2.StartControlRecording();
		Robo2.Wait(5);

		if (!Robo2.InitialLeakCurrentCheck(MIN_INITIAL_LEAKCURRENT, MAX_INITIAL_LEAKCURRENT))
		{
		Robo2.Log("Proceeding with next oocyte");
		Robo2.ValvePumpOff();
		Robo2.CloseAllValves();
		Robo2.WastePumpOff();
		continue;
		}

// Script continues here if Initial Leak current was ok
	
		ControlDisplay.SetYAxis_I(MIN_LEAKCURRENT, MAX_LEAKCURRENT)

// Starting Perfusion from valve 1
	
		Robo2.Log("Starting Perfusion ...");
	
		Robo2.Wait(5)
		Robo2.OpenValve(1);
		Robo2.ValvePumpOn(pumpspeed);
		Robo2.Wait(5);

// Starting Final Leak Current Check with perfusion
	
		
		Robo2.Log("Final leak current check (maximum = " + MIN_LEAKCURRENT + " nA) ...");
	
		if (!Robo2.LeakCurrentCheck(MIN_LEAKCURRENT, MAX_LEAKCURRENT, LEAKCURRENT_ATTEMPTS, LEAKCURRENT_WAIT))
		{
		Robo2.Log("leak current not in range");
		Robo2.ValvePumpOff();
		Robo2.CloseAllValves();
		Robo2.WastePumpOff();
		continue;
		}

// Script continues here if Final Leak current was ok

		Robo2.StopControlRecording();

		Robo2.Log("... Leak current with perfusion OK = " + Robo2.VALUE + " nA.");
	    	Robo2.Log("    ");
		Robo2.Wait(5)
	

// ***********************************************************************************************
// Protocol specific part starts here
// ***********************************************************************************************
		
	// --------------------------------------------------------------------
	// Expression test
	// --------------------------------------------------------------------
	
	Robo2.Log("Starting expression test with reference compound from valve " + reference + " ");
	
	RecDisplay.SetXAxis(0, pre_agonist_s + agonist_s + recorded_washout_s);
	RecDisplay.TrackYMax(true);
	RecDisplay.TrackYMin(true);
	Robo2.SetBaselineROI(pre_agonist_s - 3, pre_agonist_s - 1);
	Robo2.SetAnalysisROI(pre_agonist_s, pre_agonist_s + agonist_s);
	
	ControlDisplay.SetXAxis(0, pre_agonist_s + agonist_s + recorded_washout_s)
	ControlDisplay.SetYAxis_I(-10000, 100)

	// Application of reference compound
	
	Robo2.StartRecording();
	Robo2.TransmitRecording(REC_TAG_REF_COMPOUND);
	Robo2.TransmitCompoundValve(reference);
	Robo2.Wait(pre_agonist_s);
	Robo2.OpenValve(reference);
	Robo2.Wait(agonist_s);
	Robo2.OpenValve(1);
	Robo2.Wait(recorded_washout_s);
	Robo2.StopRecording();
	
	Robo2.Log(" ");
	Robo2.Log("Baseline current before application = " + Robo2.BASELINE_AVERAGE + " nA");
	signal = Robo2.MINIMUM - Robo2.BASELINE_AVERAGE
	Robo2.Log("Response to reference concentration from valve " + reference + " = " + signal + " nA");
	Robo2.Log(" ");
	
			
	/// Comparing response with user defined minimum rsponse 
	
	if (signal > minResponse || signal < maxResponse)
	{
		Robo2.Log("Response (" + signal + ") not in Range (" + minResponse + ", " + maxResponse + "), go to next oocyte");
		Robo2.Wait(2);
		
		Robo2.ValvePumpOff();
		Robo2.CloseAllValves();
		Robo2.WastePumpOff();
		continue;
	}
	
	ControlDisplay.SetXAxis(0, nonrecorded_washout_s + recorded_washout_s)
	Robo2.StartControlRecording();
	Robo2.Wait(nonrecorded_washout_s + recorded_washout_s - 10);
	Robo2.StopControlRecording();
	
	
				
	// --------------------------------------------------------------------
	// Start of Dose-Response protocol-----------------------------------------
	// --------------------------------------------------------------------
	
	Robo2.Log("Starting dose response curve with liquids in valves " + first_compound + " to " + last_compound + " ...");
	
	ControlDisplay.SetXAxis(0, pre_agonist_s + agonist_s + recorded_washout_s + nonrecorded_washout_s)
	
	RecDisplay.SetXAxis(0, pre_agonist_s + agonist_s + recorded_washout_s);
	RecDisplay.TrackYMax(true);
	RecDisplay.TrackYMin(true);
	Robo2.SetBaselineROI(pre_agonist_s - 3, pre_agonist_s - 1);
	Robo2.SetAnalysisROI(pre_agonist_s, pre_agonist_s + agonist_s);
	
	
	for (var valve = first_compound; valve <= last_compound; valve++)
	{
		Robo2.StartControlRecording();
	
		if (!Robo2.LeakCurrentCheck(MIN_LEAKCURRENT, MAX_LEAKCURRENT, LEAKCURRENT_ATTEMPTS, LEAKCURRENT_WAIT))
		{
			Robo2.Log("leak current not in range - next oocyte");
			Robo2.ValvePumpOff();
			Robo2.CloseAllValves();
			Robo2.WastePumpOff();
			
			break;
		}
		Robo2.StopControlRecording();

		Robo2.Log("... Leak current still OK = " + Robo2.VALUE + " nA - proceeding with protocol");
		Robo2.Wait(2)
		
		
		Robo2.StartRecording();
		Robo2.Wait(pre_agonist_s);
		Robo2.OpenValve(valve);
		Robo2.TransmitRecording(REC_TAG_COMPOUND)
		Robo2.TransmitCompoundValve(valve);
		Robo2.Wait(agonist_s);
		Robo2.OpenValve(1);
		Robo2.Wait(recorded_washout_s);
		Robo2.StopRecording();
		
		Robo2.Log(" ");
		Robo2.Log("Baseline current before application = " + Robo2.BASELINE_AVERAGE + " nA");
		signal = Robo2.MINIMUM - Robo2.BASELINE_AVERAGE
		Robo2.Log("Response to compound from valve " + valve + " = " + signal + " nA");
		Robo2.Log(" ");
		
		ControlDisplay.SetXAxis(0, nonrecorded_washout_s+1)
		Robo2.StartControlRecording();
		Robo2.Wait(nonrecorded_washout_s);
		Robo2.StopControlRecording();
	}

// ***********************************************************************************************
// Protocol specific part ends here
// ***********************************************************************************************
	
	Robo2.ValvePumpOff();
	Robo2.CloseAllValves();
	Robo2.WastePumpOff();
}

// -----------------------------------------------------------------------------------------------------
// End of Oocyte Loop ----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------

// Cleaning up

Robo2.ValvePumpOff();
Robo2.CloseAllValves();
Robo2.WastePumpOff();
Robo2.Log("Recording protocol completed.");
Robo2.MoveToHomePos();
